@@ -26,6 +26,8 @@ import sys
 import tarfile
 import tempfile
 
+from debian.changelog import Version
+
 from ....errors import BzrError, NoSuchFile
 from .... import osutils
 from ....export import export
@@ -71,6 +73,15 @@ class UpstreamSource(object):
         """
         raise NotImplementedError(self.get_latest_version)
 
+    def get_recent_versions(self, package, since_version=None):
+        """Retrieve recent version strings.
+
+        :param package: Name of the package
+        :param version: Last upstream version since which to retrieve versions
+        :return: Iterator over version strings
+        """
+        raise NotImplementedError(self.get_recent_versions)
+
     def version_as_revisions(self, package, version, tarballs=None):
         """Lookup the revision ids for a particular version.
 
@@ -103,55 +114,50 @@ class UpstreamSource(object):
         """
         raise NotImplementedError(self.fetch_tarballs)
 
-    def _tarball_path(self, package, version, component, target_dir, format=None):
-        return os.path.join(target_dir, tarball_name(package, version, component,
-                    format=format))
+    def _tarball_path(self, package, version, component, target_dir,
+                      format=None):
+        return os.path.join(
+            target_dir,
+            tarball_name(package, version, component, format=format))
 
 
 class AptSource(UpstreamSource):
     """Upstream source that uses apt-source."""
 
     def fetch_tarballs(self, package, upstream_version, target_dir,
-            _apt_pkg=None, components=None):
+                       _apt_pkg=None, components=None):
         if _apt_pkg is None:
             import apt_pkg
         else:
             apt_pkg = _apt_pkg
         apt_pkg.init()
 
-        def get_fn(obj, new_name, old_name):
-            try:
-                return getattr(obj, new_name)
-            except AttributeError:
-                return getattr(obj, old_name)
-
         # Handle the case where the apt.sources file contains no source
         # URIs (LP:375897)
         try:
-            get_sources = get_fn(apt_pkg, 'SourceRecords',
-                "GetPkgSrcRecords")
-            sources = get_sources()
+            sources = apt_pkg.SourceRecords()
         except SystemError:
             raise PackageVersionNotPresent(package, upstream_version, self)
 
-        restart = get_fn(sources, 'restart', 'Restart')
-        restart()
+        sources.restart()
         note("Using apt to look for the upstream tarball.")
-        lookup = get_fn(sources, 'lookup', 'Lookup')
-        while lookup(package):
-            version = get_fn(sources, 'version', 'Version')
+        while sources.lookup(package):
             filenames = []
             for (checksum, size, filename, filekind) in sources.files:
                 if filekind != "tar":
                     continue
                 filename = os.path.basename(filename)
-                if filename.startswith("%s_%s.orig" % (package, upstream_version)):
+                if filename.startswith(
+                        "%s_%s.orig" % (package, upstream_version)):
                     filenames.append(filename)
             if filenames:
-                if self._run_apt_source(package, version, target_dir):
+                if self._run_apt_source(package, sources.version, target_dir):
                     return [os.path.join(target_dir, filename)
                             for filename in filenames]
-        note("apt could not find the needed tarball.")
+                else:
+                    note("apt found %s/%s but could not download.",
+                         package, sources.version)
+        note("apt could not find %s/%s.", package, upstream_version)
         raise PackageVersionNotPresent(package, upstream_version, self)
 
     def _get_command(self, package, version_str):
@@ -178,8 +184,9 @@ class UScanError(BzrError):
 class UScanSource(UpstreamSource):
     """Upstream source that uses uscan."""
 
-    def __init__(self, tree, top_level):
+    def __init__(self, tree, subpath=None, top_level=False):
         self.tree = tree
+        self.subpath = subpath
         self.top_level = top_level
 
     def _export_file(self, name, directory):
@@ -187,6 +194,8 @@ class UScanSource(UpstreamSource):
             file = name
         else:
             file = 'debian/' + name
+        if self.subpath:
+            file = osutils.pathjoin(self.subpath, file)
         if not self.tree.has_filename(file):
             raise NoSuchFile(file, self.tree)
         output_path = os.path.join(directory, 'debian', name)
@@ -263,6 +272,9 @@ class UScanSource(UpstreamSource):
             shutil.rmtree(tmpdir)
         return self._xml_report_extract_upstream_version(text)
 
+    def get_recent_versions(self, package, since_version=None):
+        raise NotImplementedError(self.get_recent_versions)
+
     def fetch_tarballs(self, package, version, target_dir, components=None):
         note("Using uscan to look for the upstream tarball.")
         tmpdir = tempfile.mkdtemp()
@@ -296,8 +308,8 @@ class SelfSplitSource(UpstreamSource):
     def _split(self, package, upstream_version, target_filename):
         tmpdir = tempfile.mkdtemp(prefix="builddeb-get-orig-source-")
         try:
-            export_dir = os.path.join(tmpdir,
-                    "%s-%s" % (package, upstream_version))
+            export_dir = os.path.join(
+                tmpdir, "%s-%s" % (package, upstream_version))
             export(self.tree, export_dir, format="dir")
             shutil.rmtree(os.path.join(export_dir, "debian"))
             tar = tarfile.open(target_filename, "w:gz")
@@ -310,7 +322,7 @@ class SelfSplitSource(UpstreamSource):
 
     def fetch_tarballs(self, package, version, target_dir, components=None):
         note("Using the current branch without the 'debian' directory "
-                "to create the tarball")
+             "to create the tarball")
         tarball_path = self._tarball_path(package, version, None, target_dir)
         self._split(package, version, tarball_path)
         return [tarball_path]
@@ -331,8 +343,8 @@ class StackedUpstreamSource(UpstreamSource):
     def fetch_tarballs(self, package, version, target_dir, components=None):
         for source in self._sources:
             try:
-                paths = source.fetch_tarballs(package, version, target_dir,
-                    components)
+                paths = source.fetch_tarballs(
+                    package, version, target_dir, components)
             except PackageVersionNotPresent:
                 pass
             else:
@@ -349,6 +361,27 @@ class StackedUpstreamSource(UpstreamSource):
                 pass
         return None
 
+    def get_recent_versions(self, package, since_version=None):
+        versions = set()
+        for source in self._sources:
+            for version in source.get_recent_versions(package, since_version):
+                versions.add(version)
+        return list(sorted(versions))
+
+    def version_as_revisions(self, package, version, tarballs=None):
+        for source in self._sources:
+            try:
+                return source.version_as_revisions(package, version, tarballs)
+            except PackageVersionNotPresent:
+                pass
+        raise PackageVersionNotPresent(package, version, self)
+
+    def has_version(self, package, version, tarballs=None):
+        for source in self._sources:
+            if source.has_version(package, version, tarballs):
+                return True
+        return False
+
 
 def gather_orig_files(package, version, path):
     """Grab the orig files for a particular package.
@@ -420,15 +453,15 @@ class UpstreamProvider(object):
         in_target = self.already_exists_in_target(target_dir)
         if in_target is not None:
             note("Upstream tarball already exists in build directory, "
-                    "using that")
+                 "using that")
             return [(p, component_from_orig_tarball(p,
                 self.package, self.version)) for p in in_target]
         if self.already_exists_in_store() is None:
             if not os.path.exists(self.store_dir):
                 os.makedirs(self.store_dir)
             try:
-                paths = self.source.fetch_tarballs(self.package,
-                    self.version, self.store_dir)
+                paths = self.source.fetch_tarballs(
+                    self.package, self.version, self.store_dir)
             except PackageVersionNotPresent:
                 raise MissingUpstreamTarball(self.package, self.version)
             assert isinstance(paths, list)
@@ -499,6 +532,10 @@ class TarfileSource(UpstreamSource):
         repack_tarball(self.path, dest_name, target_dir=target_dir)
         return [os.path.join(target_dir, dest_name)]
 
+    def get_recent_versions(self, package, since_version=None):
+        latest_version = self.get_latest_version(package, since_version)
+        return [latest_version]
+
     def get_latest_version(self, package, version):
         if self.version is not None:
             return self.version
@@ -539,6 +576,7 @@ class LaunchpadReleaseFileSource(UpstreamSource):
             self.project = project
 
     def fetch_tarballs(self, package, version, target_dir, components=None):
+        note("Retrieving tarball for %s from Launchpad.", package)
         release = self.project.getRelease(version=version)
         if release is None:
             raise PackageVersionNotPresent(package, version, self)
@@ -570,9 +608,18 @@ class LaunchpadReleaseFileSource(UpstreamSource):
         finally:
             shutil.rmtree(tmpdir)
 
-    def get_latest_version(self, package, version):
-        versions = []
+    def _all_versions(self):
         for release in self.project_series.releases:
-            versions.append((release.date_released, release.version))
+            yield (release.date_released, release.version)
+
+    def get_recent_versions(self, package, since_version=None):
+        versions = []
+        for version in self._all_versions():
+            if since_version is None or since_version < version:
+                versions.append(version)
+        return sorted(versions)
+
+    def get_latest_version(self, package, version):
+        versions = list(self._all_versions())
         versions.sort()
         return versions[-1][1]
