@@ -27,6 +27,7 @@ from ....branch import (
     )
 from ....bzr import branch as bzr_branch
 from ....errors import (
+    GhostRevisionsHaveNoRevno,
     InvalidRevisionId,
     InvalidRevisionSpec,
     NoSuchRevision,
@@ -88,19 +89,23 @@ def _upstream_branch_version(revhistory, upstream_revision, reverse_tag_dict, pa
     if upstream_revision == NULL_REVISION:
         # No new version to merge
         return previous_version
-    for r in revhistory:
-        if r in reverse_tag_dict:
-            # If there is a newer version tagged in branch,
-            # convert to upstream version
-            # return <upstream_version>+bzr<revno>
-            for tag in reverse_tag_dict[r]:
-                upstream_version = upstream_tag_to_version(
-                        tag, package=package)
-                if upstream_version is not None:
-                    if r != upstream_revision:
-                        upstream_version = add_rev(
-                            str(upstream_version), upstream_revision)
-                    return upstream_version
+    try:
+        for r in revhistory:
+            if r in reverse_tag_dict:
+                # If there is a newer version tagged in branch,
+                # convert to upstream version
+                # return <upstream_version>+bzr<revno>
+                for tag in reverse_tag_dict[r]:
+                    upstream_version = upstream_tag_to_version(
+                            tag, package=package)
+                    if upstream_version is not None:
+                        if r != upstream_revision:
+                            upstream_version = add_rev(
+                                str(upstream_version), upstream_revision)
+                        return upstream_version
+    except RevisionNotPresent:
+        # Ghost revision somewhere on mainline.
+        pass
     return add_rev(str(previous_version), upstream_revision)
 
 
@@ -129,7 +134,8 @@ def extract_svn_revno(rev):
 
     :param rev: Revision object
     :return: Revision number, None if this was not a Subversion revision or
-         if the revision number could not be determined (bzr-svn not available).
+         if the revision number could not be determined
+         (bzr-svn not available).
     """
     try:
         from ...svn import extract_svn_foreign_revid
@@ -152,12 +158,16 @@ def upstream_version_add_revision(upstream_branch, version_string, revid):
     :param version_string: Original version string
     :param revid: Revision id of the revision
     """
-    revno = upstream_branch.revision_id_to_dotted_revno(revid)
-    revno_str = '.'.join(map(str, revno))
+    try:
+        revno = upstream_branch.revision_id_to_dotted_revno(revid)
+    except GhostRevisionsHaveNoRevno:
+        pass
+    else:
+        revno_str = '.'.join(map(str, revno))
 
-    m = re.match(r"^(.*)([\+~])bzr(\d+)$", version_string)
-    if m:
-        return "%s%sbzr%s" % (m.group(1), m.group(2), revno_str)
+        m = re.match(r"^(.*)([\+~])bzr(\d+)$", version_string)
+        if m:
+            return "%s%sbzr%s" % (m.group(1), m.group(2), revno_str)
 
     rev = upstream_branch.repository.get_revision(revid)
     gitid = extract_gitid(rev)
@@ -440,10 +450,30 @@ class UpstreamBranchSource(UpstreamSource):
             raise MultipleUpstreamTarballsNotSupported()
         return {None: self.version_as_revision(package, version, tarballs)}
 
+    def has_version(self, package, version, tarballs=None):
+        try:
+            self.version_as_revision(package, version, tarballs)
+        except PackageVersionNotPresent:
+            return False
+        else:
+            return True
+
     def get_latest_version(self, package, current_version):
         return self.get_version(
             package, current_version, self.upstream_branch.last_revision())
 
+    def get_recent_versions(self, package, since_version=None):
+        versions = []
+        tags = self.upstream_branch.tags.get_tag_dict()
+        for tag, revision in tags.items():
+            version = upstream_tag_to_version(tag, package)
+            if version is None:
+                continue
+            if since_version is not None and version <= since_version:
+                continue
+            versions.append(version)
+        return sorted(versions)
+
     def get_version(self, package, current_version, revision):
         with self.upstream_branch.lock_read():
             return upstream_branch_version(
@@ -454,6 +484,8 @@ class UpstreamBranchSource(UpstreamSource):
         if components is not None and components != [None]:
             # Multiple components are not supported
             raise PackageVersionNotPresent(package, version, self)
+        note("Looking for upstream %s/%s in upstream branch %r.",
+             package, version, self.upstream_branch)
         with self.upstream_branch.lock_read():
             if revisions is not None:
                 revid = revisions[None]
@@ -461,13 +493,18 @@ class UpstreamBranchSource(UpstreamSource):
                 revid = self.version_as_revision(package, version)
                 if revid is None:
                     raise PackageVersionNotPresent(package, version, self)
-            note("Exporting upstream branch revision %s to create the tarball",
-                 revid)
             target_filename = self._tarball_path(
                 package, version, None, target_dir)
             tarball_base = "%s-%s" % (package, version)
             rev_tree = self.upstream_branch.repository.revision_tree(revid)
-            export(rev_tree, target_filename, 'tgz', tarball_base)
+            try:
+                export(rev_tree, target_filename, 'tgz', tarball_base)
+            except UnsupportedOperation as e:
+                note('Not exporting revision from upstream branch: %s', e)
+                raise PackageVersionNotPresent(package, version, self)
+            else:
+                note("Exporting upstream branch revision %s to create "
+                     "the tarball", revid)
         return [target_filename]
 
     def __repr__(self):
@@ -498,3 +535,23 @@ class LazyUpstreamBranchSource(UpstreamBranchSource):
     def __repr__(self):
         return "<%s for %r>" % (
             self.__class__.__name__, self.upstream_branch_url)
+
+
+class LocalUpstreamBranchSource(UpstreamBranchSource):
+    """Upstream branch source in a local branch."""
+
+    def __init__(self, local_branch):
+        self.local_branch = local_branch
+        self.upstream_revision_map = {}
+        self.config = None
+
+    @classmethod
+    def from_controldir(cls, controldir):
+        return cls(controldir.open_branch('upstream'))
+
+    @property
+    def upstream_branch(self):
+        return self.local_branch
+
+    def __repr__(self):
+        return "%s(%r)" % (self.__class__.__name__, self.local_branch)
